﻿// FlashyWrappers 2.4 webcam recording (C) 2015 Pavel Langweil
// -----------------------------------------------------------
// This demo shows local webcam & microphone recording in Flash / AIR for Windows, Mac, iOS and Android.
// You can capture any DisplayObject or Stage3D with FlashyWrappers - webcam is only one of the options.
// On mobile, it is recommended to capture fullscreen (that captures both DisplayObjects and Stage3D), which is much faster there.
// Also on mobile, VideoTexture is recommended for capturing videos (StageVideo won't capture and Video is typically too slow).

// ANDROID: Your video will be saved in File.applicationStorageDirectory, which is not easily accessible in Android file browsers
//          To quickly view your video, you can also view the temp mp4 file stored in sdcard's "Android/data/air.com.rainbowcreatures.example/files/video_merged.mp4"

package  {

	import flash.display.MovieClip;
	import flash.net.FileReference;
	import flash.utils.ByteArray;
	import flash.events.Event;
	import flash.events.MouseEvent;	
	import flash.events.IOErrorEvent;		
	import flash.media.Video;
	import flash.media.Camera;
	import fl.controls.ProgressBar;

	// AIR only
	if (CONFIG::AIR) {
        import flash.filesystem.File;
        import flash.filesystem.FileMode;
        import flash.filesystem.FileStream;
        import flash.media.CameraPosition;
	}

	import com.rainbowcreatures.*;
	import com.rainbowcreatures.swf.*;	

	import flash.events.StatusEvent;
	
	[SWF(width=800,height=600, frameRate='60')]	
	public class Document extends MovieClip {
		
		// app recording state
		private var mode:String = "record";
		private var state:String = "none";
		private var extension:String = "mp4";

		private var _video:Video;
		
		private const w:int = 800;
		private const h:int = 600;

		private var _UI:UI;
		private var myEncoder:FWVideoEncoder;
				
		public function Document() {			
			_UI = new UI();
			addChild(_UI);							
			addEventListener(Event.ADDED_TO_STAGE, init);					
		}		

		private function init(e:Event):void {											
			var fps:FPSCounter = new FPSCounter();
			fps.y = 580;
			addChild(fps);
			_UI.saveDialog.button_OK.addEventListener(MouseEvent.CLICK, onSaveOKClick);
			
			// Always hand over the root DisplayObject in the getInstance constructor
			myEncoder = FWVideoEncoder.getInstance(this);			
			myEncoder.addEventListener(StatusEvent.STATUS, onStatus);						
			
			_UI.encoding_status.visible = false;			
			_UI.icon_record.visible = false;			
			_UI.button_save.buttonMode = true;
			_UI.button_save.addEventListener(MouseEvent.CLICK, onSaveClick);			
			
			// add webcam view
			_video = new Video(w, h);
			_UI.bg.addChild(_video);
            //_video.rotation = 90; 						
			//_video.x += 700;
			var _camera:Camera = tryGetFrontCamera();
			if (_camera == null) {
				showSaveDialog("You can't run this sample without a camera :)");				
			} else {
				_camera.setMode(w, h, 25);
				_camera.setQuality(0, 100);
				_video.attachCamera(_camera);
				_video.parent.setChildIndex(_video, 2);			
			}
		}
				
		// reinitialize recording (notice the myEncoder.init is there)
		private function initRecord():void {						
		
			// here we load the encoder
			// for 2.4 we also try to unload the SWF before loading it again
			// this new flow ensures the domainmemory will get freed after each recording preventing any 
			// undesired RAM accumulation caused by memory fragmentation and / or leaks
			
			if (CONFIG::FLASH_CONTAINER == 'mp4') {				
				myEncoder.load("../../../lib/FlashPlayer/mp4/");
			} else {
				myEncoder.load("../../../lib/FlashPlayer/");
				// save proper extension
				if (myEncoder.platform == FWVideoEncoder.PLATFORM_FLASH) extension = 'ogv';
			}
		}

		// FlashyWrappers status event handler
		private function onStatus(e:StatusEvent):void {
			trace("Got status event: " + e.code);
			// the encoder class is ready for frames
			if (e.code == "ready") {
				// For Flash, scale down the video dimensions by half for faster recoding. Keep in mind that encoding videos in Flash 
				// is really slow, in fact multiple times slower than recoding on mobile devices with HW accelerated video encoding.
				// If you don't need to be in web browser, always target AIR Windows / AIR Mac which can take advantage of FW native 
				// extensions (ANE's).

				if (myEncoder.platform == FWVideoEncoder.PLATFORM_FLASH) {
					myEncoder.setDimensions(400, 300);
				}
				myEncoder.start(20, FWVideoEncoder.AUDIO_MICROPHONE);
			}
			
			// FW was started
			if (e.code == "started") {
				// add frame listener - start sending frames
				addEventListener(Event.ENTER_FRAME, onFrame);
			}
												
			// video was encoded
			if (e.code == "encoded") {				
				// enable the button
				_UI.encoding_status.visible = false;			
				_UI.button_save.buttonMode = true;			
				_UI.button_save.enabled = true;								
				// set the button to "save" mode because we can save the video now
				state = "none";				
				mode = "save";		
			}							
			
			// encoding was stopped (handle this only on iOS currently, Android handler coming soon)
			if (e.code == "stopped") {
				showSaveDialog("Encoding was forced to stop. Please try again.");				
				_UI.button_save.buttonMode = true;			
				_UI.button_save.enabled = true;								
				_UI.encoding_status.visible = false;														
				_UI.icon_record.visible = false;							
				_UI.button_save.label = "Start recording";												
				// prepare for another recording
				state = "none";
				mode = "record";
			}

			// iOS only, Android coming soon			
			if (e.code == "gallery_saved") {
				prepareForRecording();
			}

			// iOS only, Android coming soon						
			if (e.code == "encoding_cancel") {
				
			}
			
			// iOS only, Android coming soon			
			if (e.code == "gallery_failed") {
				showSaveDialog("Saving to camera roll has failed.");				
				trace("Saving to camera roll has failed, permissions for access to camera roll are probably not enabled correctly for this app");
				prepareForRecording();
			}
		}

		// utility function trying to get front camera, otherwise get the standard one
		private function tryGetFrontCamera():Camera {
			if (CONFIG::AIR) {
		    		var numCameras:uint = (Camera.isSupported) ? Camera.names.length : 0;
		    		for (var i:uint = 0; i < numCameras; i++) {
		        		var cam:Camera = Camera.getCamera(String(i));
		        		if (cam && cam.position == CameraPosition.FRONT) {
			            		return cam;
		        		}
	    			} 
			}
    			return Camera.getCamera();
		}

		private function onFrame(e:Event):void {			
			if (state == "record") {
				// capture frame
				// on desktop, we can capture a target displayObject (if no argument is specified, it will just capture the whole stage)
				if (myEncoder.platform_type == FWVideoEncoder.PLATFORM_TYPE_DESKTOP) {
					myEncoder.capture(_video);				
				} else {
					// on mobile, without argument we capture fullscreen in OpenGL accelerated mode
					// you can still use the desktop mode and capture a target DisplayObject, but it will be slower
					myEncoder.capture();
				}
				
				// Display video encoding progress for our users (doesn't work precisely on iOS and Android)
				if (myEncoder.getEncodingProgress() > 0) {
					ProgressBar(_UI.encoding_status.progress).setProgress(myEncoder.getEncodingProgress(), 1);					
				}
			}
						
			// trying to finish - this is important, we must ask the encoder if its finished and can give us the data
			// don't forget in this example the encoder is running in background on another thread, so it can be delayed behind the capture phase
			if (state == "tryFinish") {
				// during this phase we also need to update the progress bar (for iOS)
				ProgressBar(_UI.encoding_status.progress).setProgress(myEncoder.getEncodingProgress(), 1);					
				
				// finish 
				myEncoder.finish();
				state = "encoding";
			}					
			
			// show encoding progress 
			if (state == "encoding") {
				ProgressBar(_UI.encoding_status.progress).setProgress(myEncoder.getEncodingProgress(), 1);		
			}
		}

		// save button click handler
		private function onSaveClick(e:MouseEvent):void {
			if (mode == "record") {
				initRecord();				
				state = "record";
				_UI.icon_record.visible = true;
				_UI.encoding_status.visible = true;
				_UI.button_save.label = "Stop recording";
				mode = "stop";
			} else {
				if (mode == "stop") {
					_UI.icon_record.visible = false;					
					_UI.button_save.label = "Save video";					
					_UI.button_save.enabled = false;													
					state = "tryFinish";				
				}
				if (mode == "save") {
					// get the final video
					var bo:ByteArray = myEncoder.getVideo();				
					trace("Saving " + bo.length + " bytes...");
					_UI.encoding_status.status.text = "Saving...";
					if (myEncoder.platform != FWVideoEncoder.PLATFORM_IOS && myEncoder.platform != FWVideoEncoder.PLATFORM_ANDROID) {
						var saveFile:FileReference = new FileReference();
						saveFile.addEventListener(Event.COMPLETE, saveCompleteHandler);
						saveFile.addEventListener(IOErrorEvent.IO_ERROR, saveIOErrorHandler);
						saveFile.save(bo, "video." + extension);												
					} else {
						// AIR only
						if (CONFIG::AIR) {										
							var file:File;
							if (myEncoder.platform == FWVideoEncoder.PLATFORM_IOS || myEncoder.platform == FWVideoEncoder.PLATFORM_ANDROID) {
								myEncoder.saveToGallery("video.mp4", "FW camera test");				
								showSaveDialog("Your gallery / camera roll");
							} 
							_UI.button_save.enabled = false;																			
						}				
					}
					
					// on mobile we saved to gallery and so there we'll need to wait for the event telling us its done
					// otherwise we can proceed
					if (myEncoder.platform != "IOS" && myEncoder.platform != "ANDROID") {
						prepareForRecording();
					}					
				}
			}
		}
		
		private function onSaveOKClick(e:MouseEvent):void {
			_UI.saveDialog.visible = false;
		}
		
		private function showSaveDialog(saveText:String):void {
			_UI.saveDialog.visible = true;
			_UI.saveDialog.location.text = saveText;
		}
		
		private function prepareForRecording():void {
			ProgressBar(_UI.encoding_status.progress).setProgress(0, 1);					
			_UI.encoding_status.status.text = "Encoding...";
			_UI.button_save.enabled = true;
			_UI.button_save.label = "Start recording";								
			mode = "record";			
		}
		
		// file was saved
		private function saveCompleteHandler(e:Event):void {
			trace("File was saved!");
			showSaveDialog("Wherever you browsed to.");
		}
		
		// some error happened, oops
		private function saveIOErrorHandler(e:IOErrorEvent):void {
			trace("Error while saving the video!");
			_UI.encoding_status.status.text = "Error!";
			_UI.button_save.enabled = false;
		}		
	}
}