﻿/**
 *
 * Hungry Hero Game
 * http://www.hungryherogame.com
 * 
 * Copyright (c) 2012 Hemanth Sharma (www.hsharma.com). All rights reserved.
 * 
 * This ActionScript source code is free.
 * You can redistribute and/or modify it in accordance with the
 * terms of the accompanying Simplified BSD License Agreement.
 *  
 */

// FlashyWrappers note: Look for "myEncoder" and "mySoundMixer" in this file to get a quick idea how to integrate with Starling.

// Note: This example is meant mainly for mobile. Capturing Stage3D on desktop / Flash is possible, but not recommended currently because its slow and that can create bad side effects.
// For Flash it is not recommended at all, unless the Stage3D is small enough area or the fps is very low.

package
{
	import flash.display.Sprite;
	import flash.display.Stage3D;
	import flash.events.Event;
	import flash.events.StatusEvent;
	import flash.geom.Rectangle;
	import flash.display.StageScaleMode;	
	import flash.display.StageAlign;
	import flash.utils.ByteArray;
	import flash.net.FileReference;
	
		
	import starling.utils.RectangleUtil;
	import starling.core.Starling;
	import starling.utils.ScaleMode;
	
	import com.rainbowcreatures.*;
	import com.rainbowcreatures.swf.*;
		
	/**
	 * SWF meta data defined for iPad 1 & 2 in landscape mode. 
	 */	
	[SWF(frameRate="60", width="1024", height="768", backgroundColor="0x000000")]
	
	/**
	 * This is the main class of the project. 
	 * 
	 * @author hsharma
	 * 
	 */
	public class HungryHero1 extends Sprite
	{		
		/** Starling object. */
		private var myStarling:Starling;
		private var mStage3D:Stage3D;		
		
		/** FlashyWrappers */
		private var myEncoder:FWVideoEncoder;
		private var mySoundMixer:FWSoundMixer;
		
		// count frames so we don't grab all those frames in Stage3D on desktop (where currently its slow)
		var frame:Number = 0;

		public function HungryHero1()
		{
			super();			
			this.addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			stage.addEventListener(Event.RESIZE, onResize);
		}
		
		private function onResize(e:Event):void {
			if (mStage3D != null && mStage3D.context3D != null) {
				mStage3D.context3D.configureBackBuffer(stage.stageWidth, stage.stageHeight, 0, false);
			}
		}
		
		/**
		 * On added to stage. 
		 * @param event
		 * 
		 */
		protected function onAddedToStage(event:Event):void
		{			
			this.removeEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
			myEncoder = FWVideoEncoder.getInstance(this);
			myEncoder.addEventListener(StatusEvent.STATUS, onEncoderStatus);
			myEncoder.load();
		}

		/**
		 * On FlashyWrappers encoder status.
		 * @param event
		 * 
		 */
		private function onEncoderStatus(event:StatusEvent):void {
			// video encoder ready
			if (event.code == 'ready') {				
				mySoundMixer = FWSoundMixer.getInstance();
				mySoundMixer.addEventListener(StatusEvent.STATUS, onMixerStatus);
				mySoundMixer.load("../../lib/FlashPlayer/");	
			}
			if (event.code == 'encoded') {
				frame = 0;
				// save the video to camera roll!
				trace("Video encoded, saving!");
				if (myEncoder.platform_type == FWVideoEncoder.PLATFORM_TYPE_MOBILE) {
					myEncoder.saveToGallery("replay.mp4", "Hungry Hero");
				} else {
					var video:ByteArray = myEncoder.getVideo();
					var saveFile:FileReference = new FileReference();
					saveFile.save(video, "replay.mp4");												
				}
			}
			if (event.code == 'gallery_saved') {
				trace("Video saved to camera roll!");
			}
			if (event.code == 'gallery_failed') {
				trace("Video saving to camera roll failed.");
			}
		}

		/**
		 * On onMixerStatus encoder status.
		 * @param event
		 * 
		 */		
		private function onMixerStatus(event:StatusEvent):void {
			if (event.code == 'ready') {
				mySoundMixer.init(null, myEncoder);				
				trace("Mixer ready, requesting Context3D...");
				mStage3D = stage.stage3Ds[0];			
	           	mStage3D.addEventListener(Event.CONTEXT3D_CREATE, onContextCreated, false, 0, true);
        		mStage3D.requestContext3D();																
			}
		}
		
		/**
		 * On Context3D created.
		 * @param event
		 * 
		 */
		private function onContextCreated(event:Event):void {
			
			mStage3D.context3D.configureBackBuffer(stage.stageWidth, stage.stageHeight, 0, false);


			/*var stageWidth:int   = 1024;
			var stageHeight:int  = 768;
			
			stage.align = StageAlign.TOP;
			var viewPort:Rectangle = RectangleUtil.fit(
     			new Rectangle(0, 0, stageWidth, stageHeight),
     			new Rectangle(0, 0, stage.fullScreenWidth, stage.fullScreenHeight),
				ScaleMode.NO_BORDER);*/
 

			// Initialize Starling object.
			myStarling = new Starling(Game, stage, null, mStage3D);
			//myStarling.stage.stageWidth  = stageWidth;
			//myStarling.stage.stageHeight = stageHeight;

			// Define basic anti aliasing.
			myStarling.antiAliasing = 1;
			
			// Show statistics for memory usage and fps.
			myStarling.showStats = true;
			
			// Position stats.
			myStarling.showStatsAt("left", "bottom");
						
			// Start Starling Framework.
			myStarling.start();			

			// Initialize sounds - this is a small change from the original Hungry Hero game. We need to initialize sounds after FWSoundMixer is loaded, not before, so we need the timing of init with this method
			Sounds.init();

			addChild(new FPSCounter());

			// Need to handle Starling rendering because of custom Context3D
			addEventListener(Event.ENTER_FRAME, onEnterFrame, false, 0, true);
			
		}
		
		/**
		 * On entering frame...
		 * @param event
		 * 
		 */		
		private function onEnterFrame(event:Event):void
		{
			
			// The back buffer needs to be cleared once per frame
			mStage3D.context3D.clear();

			// Advance both Starling instances
			myStarling.nextFrame(); 		

			// After presenting the context we're capturing the video with FlashyWrappers
			// On desktop grab just every 3rd frame to speed slow grabbing of Stage3D (on mobile its fast)
			if (myEncoder.platform_type == FWVideoEncoder.PLATFORM_TYPE_DESKTOP && frame % 3 == 0 ) {
				myEncoder.capture(mStage3D.context3D);
			} else {
				myEncoder.capture();
			}
			frame++;

			// This moves the active back buffer into the foreground.			
			mStage3D.context3D.present();	

			// After presenting the context we're capturing the video with FlashyWrappers
			if (myEncoder.platform_type == FWVideoEncoder.PLATFORM_TYPE_MOBILE) {
				//myEncoder.capture();
			} 
		}
	}
}